@@ -38,8 +38,10 @@
 //! Now in the root of your project (same directory as `Cargo.toml`) add a file `build.rs`:
 //!
 //! ```ignore
-//! fn main() -> shadow_rs::SdResult<()> {
-//!     shadow_rs::new()
+//! fn main() {
+//!     ShadowBuilder::builder()
+//!         .build_pattern(BuildPattern::RealTime)
+//!         .build().unwrap();
 //! }
 //! ```
 //!
@@ -144,6 +146,10 @@
 //! pub const GIT_STATUS_FILE: &str = "* src/lib.rs (dirty)";
 //! ```
 //!
+#[cfg(feature = "metadata")]
+pub extern crate cargo_metadata;
+#[cfg(feature = "metadata")]
+pub extern crate serde_json;
 
 mod build;
 mod ci;
@@ -152,6 +158,7 @@ mod env;
 mod err;
 mod gen_const;
 mod git;
+mod hook;
 
 /// Re-exported from the is_debug crate
 pub use is_debug::*;
@@ -168,12 +175,14 @@ use std::io::Write;
 use std::path::Path;
 
 use crate::gen_const::{
-    clap_long_version_branch_const, clap_long_version_tag_const, clap_version_branch_const,
-    clap_version_tag_const, version_branch_const, version_tag_const, BUILD_CONST_CLAP_LONG_VERSION,
-    BUILD_CONST_VERSION,
+    cargo_metadata_fn, clap_long_version_branch_const, clap_long_version_tag_const,
+    clap_version_branch_const, clap_version_tag_const, version_branch_const, version_tag_const,
+    BUILD_CONST_CLAP_LONG_VERSION, BUILD_CONST_VERSION,
 };
 pub use err::{SdResult, ShadowError};
 
+pub use crate::build::{BuildPattern, ShadowBuilder};
+use crate::hook::HookExt;
 pub use {build::ShadowConst, env::*, git::*};
 
 pub trait Format {
@@ -182,6 +191,9 @@ pub trait Format {
 
 const SHADOW_RS: &str = "shadow.rs";
 
+pub const CARGO_CLIPPY_ALLOW_ALL: &str =
+    "#[allow(clippy::all, clippy::pedantic, clippy::restriction, clippy::nursery)]";
+
 /// Add a module with the provided name which contains the build information generated by `shadow-rs`.
 ///
 /// # Example
@@ -219,11 +231,27 @@ macro_rules! shadow {
 ///    shadow_rs::new()
 /// }
 /// ```
+#[deprecated(
+    since = "0.37.0",
+    note = "Please use  [`ShadowBuilder::builder`] instead"
+)]
 pub fn new() -> SdResult<()> {
-    Shadow::build(Default::default())?;
+    ShadowBuilder::builder().build()?;
     Ok(())
 }
 
+/// Since [cargo metadata](https://crates.io/crates/cargo_metadata) details about workspace
+/// membership and resolved dependencies for the current package, storing this data can result in
+/// significantly larger crate sizes. As such, the CARGO_METADATA const is disabled by default.
+///
+/// Should you choose to retain this information, you have the option to customize a deny_const
+/// object and override the `new_deny` method parameters accordingly.
+///
+#[allow(clippy::all, clippy::pedantic, clippy::restriction, clippy::nursery)]
+pub fn default_deny() -> BTreeSet<ShadowConst> {
+    BTreeSet::from([CARGO_METADATA])
+}
+
 /// Identical to [`new`], but additionally accepts a build output denylist.
 /// This list determines constants to be excluded in the build output.
 ///
@@ -243,13 +271,19 @@ pub fn new() -> SdResult<()> {
 ///    shadow_rs::new_deny(deny)
 /// }
 /// ```
+#[deprecated(
+    since = "0.37.0",
+    note = "Please use  [`ShadowBuilder::builder`] instead"
+)]
 pub fn new_deny(deny_const: BTreeSet<ShadowConst>) -> SdResult<()> {
-    Shadow::build(deny_const)?;
+    ShadowBuilder::builder().deny_const(deny_const).build()?;
     Ok(())
 }
 
 /// Identical to [`new`], but additionally accepts an output hook.
-/// The hook receives the output file of `shadow-rs`, and it can add additional entries to the output of `shadow-rs` by writing to this file.
+///
+/// The hook receives the output file of `shadow-rs`, and it can add additional entries to the
+/// output of `shadow-rs` by writing to this file.
 /// Note that since the output will be compiled as a Rust module, inserting invalid Rust code will lead to a compile error later on.
 ///
 /// # Example
@@ -268,16 +302,24 @@ pub fn new_deny(deny_const: BTreeSet<ShadowConst>) -> SdResult<()> {
 /// }
 ///
 /// ```
+///
+#[deprecated(
+    since = "0.37.0",
+    note = "Please use  [`ShadowBuilder::builder`] instead"
+)]
 pub fn new_hook<F>(f: F) -> SdResult<()>
 where
-    F: FnOnce(&File) -> SdResult<()>,
+    F: HookExt,
 {
-    let shadow = Shadow::build(Default::default())?;
-    shadow.hook(f)
+    ShadowBuilder::builder()
+        .deny_const(f.default_deny())
+        .hook(f.hook_inner())
+        .build()?;
+    Ok(())
 }
 
 /// Returns the contents of [`std::env::vars`] as an ordered map.
-pub fn get_std_env() -> BTreeMap<String, String> {
+pub(crate) fn get_std_env() -> BTreeMap<String, String> {
     let mut env_map = BTreeMap::new();
     for (k, v) in std_env::vars() {
         env_map.insert(k, v);
@@ -287,18 +329,66 @@ pub fn get_std_env() -> BTreeMap<String, String> {
 
 /// `shadow-rs` configuration.
 ///
-/// If you use the recommended utility functions [`new`], [`new_deny`], or [`new_hook`], you do not have to handle [`Shadow`] configurations themselves.
-/// However, this struct provides more fine-grained access to `shadow-rs` configuration, such as using a denylist and a hook function at the same time.
+/// This struct encapsulates the configuration for the `shadow-rs` build process. It allows for fine-grained control over
+/// various aspects of the build, including file output, build constants, environment variables, deny lists, and build patterns.
+///
+/// While it is possible to construct a [`Shadow`] instance manually, it is highly recommended to use the [`ShadowBuilder`] builder pattern structure
+/// provided by `shadow-rs`. The builder pattern simplifies the setup process and ensures that all necessary configurations are properly set up,
+/// allowing you to customize multiple aspects simultaneously, such as using a denylist and a hook function at the same time.
+///
+/// # Fields
+///
+/// * `f`: The file that `shadow-rs` writes build information to. This file will contain serialized build constants and other metadata.
+/// * `map`: A map of build constant identifiers to their corresponding `ConstVal`. These are the values that will be written into the file.
+/// * `std_env`: A map of environment variables obtained through [`std::env::vars`]. These variables can influence the build process.
+/// * `deny_const`: A set of build constant identifiers that should be excluded from the build process. This can be populated via [`ShadowBuilder::deny_const`].
+/// * `out_path`: The path where the generated files will be placed. This is usually derived from the `OUT_DIR` environment variable but can be customized via [`ShadowBuilder::out_path`].
+/// * `build_pattern`: Determines the strategy for triggering package rebuilds (`Lazy`, `RealTime`, or `Custom`). This affects when Cargo will rerun the build script and can be configured via [`ShadowBuilder::build_pattern`].
+///
+/// # Example
+///
+/// ```no_run
+/// use std::collections::BTreeSet;
+/// use shadow_rs::{ShadowBuilder, BuildPattern, CARGO_TREE, CARGO_METADATA};
+///
+/// ShadowBuilder::builder()
+///    .build_pattern(BuildPattern::RealTime)
+///    .deny_const(BTreeSet::from([CARGO_TREE, CARGO_METADATA]))
+///    .build().unwrap();
+/// ```
+///
 #[derive(Debug)]
 pub struct Shadow {
     /// The file that `shadow-rs` writes build information to.
+    ///
+    /// This file will contain all the necessary information about the build, including serialized build constants and other metadata.
     pub f: File,
+
     /// The values of build constants to be written.
+    ///
+    /// This is a mapping from `ShadowConst` identifiers to their corresponding `ConstVal` objects. Each entry in this map represents a build constant that will be included in the final build.
     pub map: BTreeMap<ShadowConst, ConstVal>,
-    /// Build environment variables, obtained through [`get_std_env`].
+
+    /// Build environment variables, obtained through [`std::env::vars`].
+    ///
+    /// These environment variables can affect the build process and are captured here for consistency and reproducibility.
     pub std_env: BTreeMap<String, String>,
-    /// Constants in the denylist, passed through [`new_deny`] or [`Shadow::build`].
+
+    /// Constants in the deny list, passed through [`ShadowBuilder::deny_const`].
+    ///
+    /// This set contains build constant identifiers that should be excluded from the build process. By specifying these, you can prevent certain constants from being written into the build file.
     pub deny_const: BTreeSet<ShadowConst>,
+
+    /// The output path where generated files will be placed.
+    ///
+    /// This specifies the directory where the build script will write its output. It's typically set using the `OUT_DIR` environment variable but can be customized using [`ShadowBuilder::out_path`].
+    pub out_path: String,
+
+    /// Determines the strategy for triggering package rebuilds.
+    ///
+    /// This field sets the pattern for how often the package should be rebuilt. Options include `Lazy`, `RealTime`, and `Custom`, each with its own implications on the build frequency and conditions under which a rebuild is triggered.
+    /// It can be configured using [`ShadowBuilder::build_pattern`].
+    pub build_pattern: BuildPattern,
 }
 
 impl Shadow {
@@ -306,7 +396,7 @@ impl Shadow {
     /// The hook function is run as well, allowing it to append to `shadow-rs`'s output.
     pub fn hook<F>(&self, f: F) -> SdResult<()>
     where
-        F: FnOnce(&File) -> SdResult<()>,
+        F: Fn(&File) -> SdResult<()>,
     {
         let desc = r#"/// Below code generated by project custom from by build.rs"#;
         writeln!(&self.f, "\n{desc}\n")?;
@@ -333,21 +423,51 @@ impl Shadow {
         CiType::None
     }
 
-    /// Create a new [`Shadow`] configuration with a provided denylist.
-    /// The project source path and output file are automatically derived from Cargo build environment variables.
-    pub fn build(deny_const: BTreeSet<ShadowConst>) -> SdResult<Shadow> {
-        let src_path = std::env::var("CARGO_MANIFEST_DIR")?;
-        let out_path = std::env::var("OUT_DIR")?;
-        Self::build_inner(src_path, out_path, deny_const)
+    /// Checks if the specified build constant is in the deny list.
+    ///
+    /// # Arguments
+    /// * `deny_const` - A value of type `ShadowConst` representing the build constant to check.
+    ///
+    /// # Returns
+    /// * `true` if the build constant is present in the deny list; otherwise, `false`.
+    pub fn deny_contains(&self, deny_const: ShadowConst) -> bool {
+        self.deny_const.contains(&deny_const)
     }
 
-    fn build_inner(
+    /// Create a new [`Shadow`] configuration with a provided denylist.
+    /// The project source path and output file are automatically derived from Cargo build environment variables.
+    #[deprecated(
+        since = "0.37.0",
+        note = "Please use  [`ShadowBuilder::builder`] instead"
+    )]
+    pub fn build(deny_const: BTreeSet<ShadowConst>) -> SdResult<Shadow> {
+        ShadowBuilder::builder().deny_const(deny_const).build()
+    }
+
+    #[deprecated(
+        since = "0.37.0",
+        note = "Please use  [`ShadowBuilder::builder`] instead"
+    )]
+    pub fn build_with(
         src_path: String,
         out_path: String,
         deny_const: BTreeSet<ShadowConst>,
     ) -> SdResult<Shadow> {
+        ShadowBuilder::builder()
+            .deny_const(deny_const)
+            .src_path(src_path)
+            .out_path(out_path)
+            .build()
+    }
+
+    fn build_inner(builder: ShadowBuilder) -> SdResult<Shadow> {
+        let out_path = builder.get_out_path()?;
+        let src_path = builder.get_src_path()?;
+        let build_pattern = builder.get_build_pattern().clone();
+        let deny_const = builder.get_deny_const().clone();
+
         let out = {
-            let path = Path::new(out_path.as_str());
+            let path = Path::new(out_path);
             if !out_path.ends_with('/') {
                 path.join(format!("{out_path}/{SHADOW_RS}"))
             } else {
@@ -360,6 +480,8 @@ impl Shadow {
             map: Default::default(),
             std_env: Default::default(),
             deny_const,
+            out_path: out_path.to_string(),
+            build_pattern,
         };
         shadow.std_env = get_std_env();
 
@@ -370,7 +492,7 @@ impl Shadow {
         for (k, v) in new_project(&shadow.std_env) {
             map.insert(k, v);
         }
-        for (k, v) in new_system_env(&shadow.std_env) {
+        for (k, v) in new_system_env(&shadow) {
             map.insert(k, v);
         }
         shadow.map = map;
@@ -380,6 +502,11 @@ impl Shadow {
 
         shadow.write_all()?;
 
+        // handle hook
+        if let Some(h) = builder.get_hook() {
+            shadow.hook(h.hook_inner())?
+        }
+
         Ok(shadow)
     }
 
@@ -403,6 +530,10 @@ impl Shadow {
     }
 
     /// Request Cargo to re-run the build script if any environment variable observed by this [`Shadow`] configuration changes.
+    #[deprecated(
+        since = "0.37.0",
+        note = "Please use  [`ShadowBuilder::build_pattern`] instead"
+    )]
     pub fn cargo_rerun_if_env_changed(&self) {
         for k in self.std_env.keys() {
             println!("cargo:rerun-if-env-changed={k}");
@@ -411,6 +542,10 @@ impl Shadow {
 
     /// Request Cargo to re-run the build script if any of the specified environment variables change.
     /// This function is not influenced by this [`Shadow`] configuration.
+    #[deprecated(
+        since = "0.37.0",
+        note = "Please use  [`ShadowBuilder::build_pattern`] instead"
+    )]
     pub fn cargo_rerun_env_inject(&self, env: &[&str]) {
         for k in env {
             println!("cargo:rerun-if-env-changed={}", *k);
@@ -418,8 +553,10 @@ impl Shadow {
     }
 
     fn gen_const(&mut self) -> SdResult<()> {
+        let out_dir = &self.out_path;
+        self.build_pattern.rerun_if(self.map.keys(), out_dir);
+
         for (k, v) in self.map.clone() {
-            println!("cargo:rerun-if-env-changed={k}");
             self.write_const(k, v)?;
         }
         Ok(())
@@ -431,7 +568,7 @@ impl Shadow {
 // Author: https://www.github.com/baoyachi
 // Generation time: {}
 "#,
-            DateTime::now().human_format()
+            DateTime::now().to_rfc2822()
         );
         writeln!(&self.f, "{desc}\n\n")?;
         Ok(())
@@ -439,31 +576,34 @@ impl Shadow {
 
     fn write_const(&mut self, shadow_const: ShadowConst, val: ConstVal) -> SdResult<()> {
         let desc = format!("#[doc=r#\"{}\"#]", val.desc);
-
         let define = match val.t {
-            ConstType::OptStr => format!(
-                "#[allow(dead_code)]\n\
-                #[allow(clippy::all)]\n\
-            pub const {} :{} = r#\"{}\"#;",
-                shadow_const.to_ascii_uppercase(),
-                ConstType::Str,
-                ""
-            ),
             ConstType::Str => format!(
                 "#[allow(dead_code)]\n\
-                #[allow(clippy::all)]\n\
+                {}\n\
             pub const {} :{} = r#\"{}\"#;",
+                CARGO_CLIPPY_ALLOW_ALL,
                 shadow_const.to_ascii_uppercase(),
                 ConstType::Str,
                 val.v
             ),
             ConstType::Bool => format!(
                 "#[allow(dead_code)]\n\
+            	{}\n\
             pub const {} :{} = {};",
+                CARGO_CLIPPY_ALLOW_ALL,
                 shadow_const.to_ascii_uppercase(),
                 ConstType::Bool,
                 val.v.parse::<bool>().unwrap()
             ),
+            ConstType::Slice => format!(
+                "#[allow(dead_code)]\n\
+            	{}\n\
+            pub const {} :{} = &{:?};",
+                CARGO_CLIPPY_ALLOW_ALL,
+                shadow_const.to_ascii_uppercase(),
+                ConstType::Slice,
+                val.v.as_bytes()
+            ),
         };
 
         writeln!(&self.f, "{desc}")?;
@@ -505,8 +645,15 @@ impl Shadow {
         let mut print_val = String::from("\n");
 
         // append gen const
-        for k in self.map.keys() {
-            let tmp = format!(r#"{}println!("{k}:{{{k}}}\n");{}"#, "\t", "\n");
+        for (k, v) in &self.map {
+            let tmp = match v.t {
+                ConstType::Str | ConstType::Bool => {
+                    format!(r#"{}println!("{k}:{{{k}}}\n");{}"#, "\t", "\n")
+                }
+                ConstType::Slice => {
+                    format!(r#"{}println!("{k}:{{:?}}\n",{});{}"#, "\t", k, "\n",)
+                }
+            };
             print_val.push_str(tmp.as_str());
         }
 
@@ -519,12 +666,15 @@ impl Shadow {
         let everything_define = format!(
             "/// Prints all built-in `shadow-rs` build constants to standard output.\n\
             #[allow(dead_code)]\n\
+            {CARGO_CLIPPY_ALLOW_ALL}\n\
             pub fn print_build_in() {\
             {{print_val}}\
             }\n",
         );
         writeln!(&self.f, "{everything_define}")?;
 
+        writeln!(&self.f, "{}", cargo_metadata_fn(self))?;
+
         Ok(())
     }
 }
@@ -536,23 +686,28 @@ mod tests {
 
     #[test]
     fn test_build() -> SdResult<()> {
-        Shadow::build_inner("./".into(), "./".into(), Default::default())?;
+        ShadowBuilder::builder()
+            .src_path("./")
+            .out_path("./")
+            .build()?;
         let shadow = fs::read_to_string("./shadow.rs")?;
         assert!(!shadow.is_empty());
         assert!(shadow.lines().count() > 0);
-        println!("{shadow}");
         Ok(())
     }
 
     #[test]
     fn test_build_deny() -> SdResult<()> {
-        let mut deny = BTreeSet::new();
-        deny.insert(CARGO_TREE);
-        Shadow::build_inner("./".into(), "./".into(), deny)?;
+        ShadowBuilder::builder()
+            .src_path("./")
+            .out_path("./")
+            .deny_const(BTreeSet::from([CARGO_TREE]))
+            .build()?;
+
         let shadow = fs::read_to_string("./shadow.rs")?;
         assert!(!shadow.is_empty());
         assert!(shadow.lines().count() > 0);
-        println!("{shadow}");
+        // println!("{shadow}");
         let expect = "pub const CARGO_TREE :&str";
         assert!(!shadow.contains(expect));
         Ok(())
